@@ -13,6 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Miscellaneous utility functions for use with Swift."""
+import gzip
 import hashlib
 import hmac
 import json
@@ -67,26 +68,43 @@ def generate_temp_url(path, seconds, key, method, absolute=False):
 
     :param path: The full path to the Swift object. Example:
         /v1/AUTH_account/c/o.
-    :param seconds: The amount of time in seconds the temporary URL will
-        be valid for.
+    :param seconds: If absolute is False then this specifies the amount of time
+        in seconds for which the temporary URL will be valid. If absolute is
+        True then this specifies an absolute time at which the temporary URL
+        will expire.
     :param key: The secret temporary URL key set on the Swift
         cluster. To set a key, run 'swift post -m
         "Temp-URL-Key: <substitute tempurl key here>"'
     :param method: A HTTP method, typically either GET or PUT, to allow
         for this temporary URL.
-    :raises: ValueError if seconds is not a positive integer
-    :raises: TypeError if seconds is not an integer
+    :param absolute: if True then the seconds parameter is interpreted as an
+        absolute Unix time, otherwise seconds is interpreted as a relative time
+        offset from current time.
+    :raises: ValueError if seconds is not a whole number or path is not to
+        an object.
     :return: the path portion of a temporary URL
     """
-    if seconds < 0:
-        raise ValueError('seconds must be a positive integer')
     try:
-        if not absolute:
-            expiration = int(time.time() + seconds)
-        else:
-            expiration = int(seconds)
-    except TypeError:
-        raise TypeError('seconds must be an integer')
+        seconds = float(seconds)
+        if not seconds.is_integer():
+            raise ValueError()
+        seconds = int(seconds)
+        if seconds < 0:
+            raise ValueError()
+    except ValueError:
+        raise ValueError('seconds must be a whole number')
+
+    if isinstance(path, six.binary_type):
+        try:
+            path_for_body = path.decode('utf-8')
+        except UnicodeDecodeError:
+            raise ValueError('path must be representable as UTF-8')
+    else:
+        path_for_body = path
+
+    parts = path_for_body.split('/', 4)
+    if len(parts) != 5 or parts[0] or not all(parts[1:]):
+        raise ValueError('path must be full path to an object e.g. /v1/a/c/o')
 
     standard_methods = ['GET', 'PUT', 'HEAD', 'POST', 'DELETE']
     if method.upper() not in standard_methods:
@@ -94,21 +112,31 @@ def generate_temp_url(path, seconds, key, method, absolute=False):
         logger.warning('Non default HTTP method %s for tempurl specified, '
                        'possibly an error', method.upper())
 
-    hmac_body = '\n'.join([method.upper(), str(expiration), path])
+    if not absolute:
+        expiration = int(time.time() + seconds)
+    else:
+        expiration = seconds
+    hmac_body = u'\n'.join([method.upper(), str(expiration), path_for_body])
 
     # Encode to UTF-8 for py3 compatibility
-    sig = hmac.new(key.encode(),
-                   hmac_body.encode(),
-                   hashlib.sha1).hexdigest()
+    if not isinstance(key, six.binary_type):
+        key = key.encode('utf-8')
+    sig = hmac.new(key, hmac_body.encode('utf-8'), hashlib.sha1).hexdigest()
 
-    return ('{path}?temp_url_sig='
-            '{sig}&temp_url_expires={exp}'.format(
-                path=path,
-                sig=sig,
-                exp=expiration))
+    temp_url = u'{path}?temp_url_sig={sig}&temp_url_expires={exp}'.format(
+        path=path_for_body, sig=sig, exp=expiration)
+    # Have return type match path from caller
+    if isinstance(path, six.binary_type):
+        return temp_url.encode('utf-8')
+    else:
+        return temp_url
 
 
 def parse_api_response(headers, body):
+    if headers.get('content-encoding') == 'gzip':
+        with gzip.GzipFile(fileobj=six.BytesIO(body), mode='r') as gz:
+            body = gz.read()
+
     charset = 'utf-8'
     # Swift *should* be speaking UTF-8, but check content-type just in case
     content_type = headers.get('content-type', '')
@@ -118,6 +146,20 @@ def parse_api_response(headers, body):
     return json.loads(body.decode(charset))
 
 
+def split_request_headers(options, prefix=''):
+    headers = {}
+    for item in options:
+        split_item = item.split(':', 1)
+        if len(split_item) == 2:
+            headers[(prefix + split_item[0]).title()] = split_item[1].strip()
+        else:
+            raise ValueError(
+                "Metadata parameter %s must contain a ':'.\n%s"
+                % (item, "Example: 'Color:Blue' or 'Size:Large'")
+            )
+    return headers
+
+
 def report_traceback():
     """
     Reports a timestamp and full traceback for a given exception.
@@ -202,27 +244,36 @@ class LengthWrapper(object):
     def __init__(self, readable, length, md5=False):
         """
         :param readable: The filelike object to read from.
-        :param length: The maximum amount of content to that can be read from
+        :param length: The maximum amount of content that can be read from
                        the filelike object before it is simulated to be
                        empty.
         :param md5: Flag to enable calculating the MD5 of the content
                     as it is read.
         """
-        self.md5sum = hashlib.md5() if md5 else NoopMD5()
+        self._md5 = md5
+        self._reset_md5()
         self._length = self._remaining = length
         self._readable = readable
+        self._can_reset = all(hasattr(readable, attr)
+                              for attr in ('seek', 'tell'))
+        if self._can_reset:
+            self._start = readable.tell()
 
     def __len__(self):
         return self._length
 
+    def _reset_md5(self):
+        self.md5sum = hashlib.md5() if self._md5 else NoopMD5()
+
     def get_md5sum(self):
         return self.md5sum.hexdigest()
 
-    def read(self, *args, **kwargs):
+    def read(self, size=-1):
         if self._remaining <= 0:
             return ''
 
-        chunk = self._readable.read(*args, **kwargs)[:self._remaining]
+        to_read = self._remaining if size < 0 else min(size, self._remaining)
+        chunk = self._readable.read(to_read)
         self._remaining -= len(chunk)
 
         try:
@@ -231,3 +282,37 @@ class LengthWrapper(object):
             self.md5sum.update(chunk.encode())
 
         return chunk
+
+    @property
+    def reset(self):
+        if self._can_reset:
+            return self._reset
+        raise AttributeError("%r object has no attribute 'reset'" %
+                             type(self).__name__)
+
+    def _reset(self, *args, **kwargs):
+        if not self._can_reset:
+            raise TypeError('%r object cannot be reset; needs both seek and '
+                            'tell methods' % type(self._readable).__name__)
+        self._readable.seek(self._start)
+        self._reset_md5()
+        self._remaining = self._length
+
+
+def iter_wrapper(iterable):
+    for chunk in iterable:
+        if len(chunk) == 0:
+            # If we emit an empty chunk, requests will go ahead and send it,
+            # causing the server to close the connection
+            continue
+        yield chunk
+
+
+def n_at_a_time(seq, n):
+    for i in range(0, len(seq), n):
+        yield seq[i:i + n]
+
+
+def n_groups(seq, n):
+    items_per_group = ((len(seq) - 1) // n) + 1
+    return n_at_a_time(seq, items_per_group)
